import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.imageio.*;

/**
 * パネルクラス
 */
public class MGPanel extends JPanel {

	/** フェーズ */
	public static final int			PHASE_TITLE			= 0;
	public static final int			PHASE_START			= 1;
	public static final int			PHASE_GAME			= 2;
	public static final int			PHASE_EXTEND		= 3;
	public static final int			PHASE_EXTEND_GAME	= 4;
	public static final int			PHASE_GAMEOVER		= 5;

	/** ピコピコハンマーの状態 */
	private final int				JOTAI_PH_FUTU		= 0;
	private final int				JOTAI_PH_TATAKU		= 1;
	private final int				JOTAI_PH_GABYO		= 2;
	private final int				JOTAI_PH_ROCK		= 3;

	/** 敵キャラの状態 */
	private final int				JOTAI_NASI			= -1;
	private final int				JOTAI_DERU			= 1;
	private final int				JOTAI_MOGURA		= 2;
	private final int				JOTAI_YARARETA		= 3;
	private final int				JOTAI_GABYO			= 4;
	private final int				JOTAI_ROCK			= 5;

	/** フェーズ */
	public		int					phase				= 0;

	/** マウスアダプタ */
	private		MGMouseAdapter		mgma				= null;

	/** ピコピコハンマー座標 */
	private		int					px					= 0;
	/** ピコピコハンマー座標 */
	private		int					py					= 0;

	/** 敵座標 */
	private		int[]				xTekis				= new int[10];
	/** 敵座標 */
	private		int[]				yTekis				= new int[10];

	/** うしし座標 */
	private		int					yUsisi				= 0;

	/** 背景 */
	private		BufferedImage		imageBackground		= null;

	/** 出る */
	private		BufferedImage[]		imageDerus			= null;

	/** ピコピコハンマー */
	private		BufferedImage[]		imagePHs			= null;
	/** ピコピコハンマー状態 */
	private		int					jotaiPH				= JOTAI_PH_FUTU;
	/** ピコピコハンマーの時間 */
	private		int					timePH				= 0;

	/** 敵キャラ */
	private		BufferedImage[]		imageTekis			= null;
	/** 敵キャラ状態 */
	private		int[]				jotaiTekis			= new int[10];
	/** 敵キャラの時間 */
	private		int[]				timeTekis			= new int[10];

	/** うししキャラ */
	private		BufferedImage		imageUsisi			= null;
	/** うししキャラの時間 */
	private		int					timeUsisi			= 0;

	/** 敵キャラの出現時間 */
	private		int					timeNext			= 0;

	/** タイマー */
	private		java.util.Timer		timerThis			= null;

	/** スタート */
	private		int					timeStart			= 180;
	private		Font				fontStart			= new Font("ＭＳ ゴシック", Font.BOLD, 60);

	/** 得点 */
	private		int					score				= 0;
	private		Font				fontScore			= new Font("ＭＳ ゴシック", Font.BOLD | Font.ITALIC, 24);

	/** 制限時間 */
	private		int					time				= 1859;
	private		Font				fontTime			= new Font("ＭＳ ゴシック", Font.BOLD, 24);

	/** 延長戦突入 */
	private		Font				fontExtraGame		= new Font("ＭＳ ゴシック", Font.BOLD, 60);

	/** ゲームオーバー */
	private		Font				fontGameOver		= new Font("ＭＳ ゴシック", Font.BOLD, 48);

	/** タイトルクラス */
	private		MGTitle				title				= null;

	/** BGM */
	private		KSoundWave			waveTitle			= null;
	private		KSoundMidi			midiMoguraDance		= null;
	private		KSoundMidi			midiMoguraDance2	= null;

	/** 効果音 */
	private		KSoundWave			waveMoguraDeru		= null;
	private		KSoundWave			wavePicoHammerMiss	= null;
	private		KSoundWave			wavePicoHammerHit	= null;
	private		KSoundWave			wavePicoHammerGabyo	= null;
	private		KSoundWave			wavePicoHammerRock	= null;
	private		KSoundWave			waveUsisi			= null;
	private		KSoundWave			waveFade			= null;
	private		KSoundWave			waveGameOver		= null;

	/**
	 * コンストラクタ
	 */
	public MGPanel() {

		// スーパークラスを呼び出す
		super();

		try{

			// パネルサイズ
			super.setPreferredSize(new Dimension(800, 600));
			// レイアウト設定
			super.setLayout(null);

			// マウスアダプタを生成
			mgma = new MGMouseAdapter();
			// パネルにマウスリスナーを追加する
			super.addMouseListener(mgma);
			super.addMouseMotionListener(mgma);

			// 背景を読み込む
			InputStream isBackground = this.getClass().getResourceAsStream("image/Background.jpg");
			imageBackground = ImageIO.read(isBackground);
			isBackground.close();

			// ピコピコハンマーを読み込む
			imagePHs = new BufferedImage[3];
			InputStream isPH00 = this.getClass().getResourceAsStream("image/PH00.gif");
			imagePHs[0] = ImageIO.read(isPH00);
			isPH00.close();

			InputStream isPH01 = this.getClass().getResourceAsStream("image/PH01.gif");
			imagePHs[1] = ImageIO.read(isPH01);
			isPH01.close();

			InputStream isPHGabyo = this.getClass().getResourceAsStream("image/PHGabyo.gif");
			imagePHs[2] = ImageIO.read(isPHGabyo);
			isPHGabyo.close();

			// 出るを読み込む
			imageDerus = new BufferedImage[2];
			InputStream isDeru00 = this.getClass().getResourceAsStream("image/Deru00.gif");
			imageDerus[0] = ImageIO.read(isDeru00);
			isDeru00.close();

			InputStream isDeru01 = this.getClass().getResourceAsStream("image/Deru01.gif");
			imageDerus[1] = ImageIO.read(isDeru01);
			isDeru01.close();

			// 敵キャラを読み込む
			imageTekis = new BufferedImage[4];
			InputStream isMogura00 = this.getClass().getResourceAsStream("image/Mogura00.gif");
			imageTekis[0] = ImageIO.read(isMogura00);
			isMogura00.close();

			InputStream isMogura01 = this.getClass().getResourceAsStream("image/Mogura01.gif");
			imageTekis[1] = ImageIO.read(isMogura01);
			isMogura01.close();

			InputStream isGabyo = this.getClass().getResourceAsStream("image/Gabyo00.gif");
			imageTekis[2] = ImageIO.read(isGabyo);
			isGabyo.close();

			InputStream isRock = this.getClass().getResourceAsStream("image/Rock00.gif");
			imageTekis[3] = ImageIO.read(isRock);
			isRock.close();

			// うししを読み込む
			InputStream isUsisi = this.getClass().getResourceAsStream("image/Usisi00.gif");
			imageUsisi = ImageIO.read(isUsisi);
			isUsisi.close();

			// 効果音を生成
			waveMoguraDeru = new KSoundWave(this, "sound/MoguraDeru.wav", false);
			wavePicoHammerMiss = new KSoundWave(this, "sound/PicoHammerMiss.wav", false);
			wavePicoHammerHit = new KSoundWave(this, "sound/PicoHammerHit.wav", false);
			wavePicoHammerGabyo = new KSoundWave(this, "sound/PicoHammerGabyo.wav", false);
			wavePicoHammerRock = new KSoundWave(this, "sound/PicoHammerRock.wav", false);
			waveUsisi = new KSoundWave(this, "sound/USS3.wav", false);
			waveFade = new KSoundWave(this, "sound/Fade.wav", false);
			waveGameOver = new KSoundWave(this, "sound/GameOver.wav", false);

			// タイトルクラス生成
			title = new MGTitle(this);

			// 初期化
			init();

		}catch(Exception ex){
			ex.printStackTrace();
			JOptionPane.showMessageDialog(this, "ERROR : " + ex.toString());
		}

	} // end MGPanel

	/**
	 * 初期化
	 */
	public void init(){

		phase = PHASE_TITLE;
		jotaiPH = JOTAI_PH_FUTU;
		score = 0;
		time = 1859;
		timeNext = 0;

		// タイトルクラスも初期化
		title.init();

		// ループ
		for(int i = 0; i < 10; i++){
			// いない
			jotaiTekis[i] = JOTAI_NASI;
		}

		// タイマーを生成
		timerThis = new java.util.Timer();

		// タイマーをスタート
		timerThis.scheduleAtFixedRate(new TimerActionListener(), 2000l, 70l);

	} // end init

	/**
	 * ゲームスタート
	 */
	public void start(){

		// タイマーをキャンセル
		timerThis.cancel();

		jotaiPH = JOTAI_PH_FUTU;

		timeStart = 180;

		// タイマーを生成
		timerThis = new java.util.Timer();

		// タイマーをスタート
		timerThis.scheduleAtFixedRate(new TimerActionListener(), 500l, 16l);

		// ゲームフェーズへ
		phase = MGPanel.PHASE_START;

	} // end start

	/**
	 * 実行
	 */
	public void run(){

		// フェーズ
		switch(phase){

			// タイトル
			case PHASE_TITLE:

				// タイトルクラスの実行メソッドを呼び出す
				title.run();

				// ブレイク
				break;

			// スタートの場合、
			case PHASE_START:

				// 時間を減らす
				timeStart--;

				// 時間が来たら、
				if(timeStart <= 0){

					// BGMを生成
					midiMoguraDance = new KSoundMidi(this, "sound/MoguraDance.mid", false);
					// BGMをスタート
					midiMoguraDance.start();

					// フェーズ変更
					phase = PHASE_GAME;

				}

				// ブレイク
				break;

			// ゲーム中
			case PHASE_GAME:
			case PHASE_EXTEND_GAME:

				// 制限時間を減らす
				time--;

				// 制限時間が来たら、
				if(time == 0){

					// 延長戦
					if(phase == PHASE_GAME && score >= 50){

						// 時間を戻す
						time = 300;
						// フェーズ変更
						phase = PHASE_EXTEND;

					// ゲームオーバー
					}else{

						// 時間を戻す
						time = 600;
						// フェーズ変更
						phase = PHASE_GAMEOVER;

					}

					// ブレイク
					break;
				}

				// 画びょう or 岩 の場合、
				if(jotaiPH == JOTAI_PH_GABYO || jotaiPH == JOTAI_PH_ROCK){

					// 時間－１
					timePH--;

					// 0.5秒経ったら
					if(timePH == 30){

						// 効果音をスタート
						waveUsisi.start();

					}

					// 時間がたった場合、
					if(timePH <= 0){

						// 状態を戻す
						jotaiPH = JOTAI_PH_FUTU;

					}

				} // end if 画びょう or 岩 の場合

				// 出現時間－１
				timeNext--;

				// 出現時間であれば、
				if(timeNext <= 0){

					// ループ
					for(int i = 0; i < 10; i++){

						// いない場合、
						if(jotaiTekis[i] == JOTAI_NASI){

							// 新しい敵を表示する
							jotaiTekis[i] = JOTAI_DERU;

							// ループ
							while(true){

								// 位置を決定
								xTekis[i] = (int)(Math.random() * 550);
								yTekis[i] = (int)(Math.random() * 400);

								// 重なりチェック
								int j = 0;

								// ループ
								for(j = 0; j < 10; j++){

									// 自分自身の場合、
									if(i == j){
										// コンティニュー
										continue;
									}

									// いる場合、
									if(jotaiTekis[j] != JOTAI_NASI){

										// 当たっている場合、
										if(xTekis[j] - 100 < xTekis[i] && xTekis[i] < (xTekis[j] + 100) &&
											yTekis[j] - 100 < yTekis[i] && yTekis[i] < (yTekis[j] + 100)){

											// ループを抜ける
											break;

										}

									} // end if いる場合

								} // end for ループ

								// 重なっていなかった場合、
								if(j >= 10){
									// ループを抜ける
									break;
								}

							} // end while ループ

							// 時間をリセット
							timeTekis[i] = 60;

							// ループを抜ける
							break;

						} // end if いない場合

					} // end for

					// 時間をリセット
					timeNext = 15 + (int)(Math.random() * 30 - ((phase == PHASE_EXTEND_GAME)?10: 0));

				} // end if 出現時間であれば

				// ループ
				for(int i = 0; i < 10; i++){

					// いない場合、
					if(jotaiTekis[i] == JOTAI_NASI){
						// コンティニュー
						continue;
					}

					// 時間－１
					timeTekis[i]--;

					// 状態により処理を分ける
					switch(jotaiTekis[i]){

						// 出る
						case JOTAI_DERU:

							// 時間が経てば
							if(timeTekis[i] <= 0){

								// 乱数
								double d = Math.random();

								// 94％
								if(d < 0.94 - ((phase == PHASE_EXTEND_GAME)?0.06: 0)){

									// もぐら
									jotaiTekis[i] = JOTAI_MOGURA;

								// 3％（6％）
								}else if(d < 0.97 - ((phase == PHASE_EXTEND_GAME)?0.03: 0)){

									// 画びょう
									jotaiTekis[i] = JOTAI_GABYO;

								// 3％（6％）
								}else{

									// 岩
									jotaiTekis[i] = JOTAI_ROCK;

								}

								// 効果音をスタート
								waveMoguraDeru.start();

								// 時間をリセット
								timeTekis[i] = 180;

							} // end if 時間が経てば

							// ブレイク
							break;

						// もぐら出現中／やられている／画びょう／岩の場合、

						case JOTAI_MOGURA:
						case JOTAI_YARARETA:
						case JOTAI_GABYO:
						case JOTAI_ROCK:

							// 時間が経てば
							if(timeTekis[i] <= 0){

								// 状態を変更
								jotaiTekis[i] = JOTAI_NASI;

							}

							// ブレイク
							break;

						default:
							// ブレイク
							break;

					} // end switch 状態により処理を分ける

				} // end for ループ

				// ブレイク
				break;

			// 時間終了
			case PHASE_EXTEND:

				// 制限時間を減らす
				time--;

				// 1.5秒経ったら
				if(time == 210){

					// 効果音をスタート
					waveFade.start();

				}

				// 制限時間が来たら、
				if(time == 0){

					// 延長戦へ
					phase = PHASE_EXTEND_GAME;

					// 時間
					time = 1859;

					// BGMを生成
					midiMoguraDance2 = new KSoundMidi(this, "sound/MoguraDance2.mid", false);
					// BGMをスタート
					midiMoguraDance2.start();

				} // end if 制限時間が来たら

				// ブレイク
				break;

			// ゲームオーバー
			case PHASE_GAMEOVER:

				// 制限時間を減らす
				time--;

				// ２秒経ったら、
				if(time == 480){

					// 効果音をスタート
					waveGameOver.start();

				}

				// 制限時間が来たら、
				if(time == 0){

					// タイマーをキャンセル
					timerThis.cancel();

					// 初期化
					init();

				}

				// ブレイク
				break;

		} // end switch

	} // end run

	/**
	 * 描画メソッド.
	 * <pre>
	 * ペイントする必要があるときにこのメソッドを呼び出されます。
	 * </pre>
	 */
	public void paint(Graphics g){

		// タイトルの場合、
		if(phase == PHASE_TITLE){

			// タイトルクラスを呼び出す
			title.paint(g);

			// リターン
			return;

		}

		// 背景を描画する
		g.drawImage(imageBackground, 0, 0, 800, 600, this);

		// スタートの場合、
		if(phase == PHASE_START){

			g.setColor(Color.yellow);
			g.setFont(fontStart);
			g.drawString("ゲームスタート", 180, 300);

		}

		// ループ
		for(int i = 0; i < 10; i++){

			// いない場合、
			if(jotaiTekis[i] == JOTAI_NASI){
				// コンティニュー
				continue;
			}

			// 状態により処理を分ける
			switch(jotaiTekis[i]){

				// 出る
				case JOTAI_DERU:

					// 敵を描画する
					g.drawImage(imageDerus[timeTekis[i] / 10 % 2], xTekis[i], yTekis[i], 100, 100, this);

					// ブレイク
					break;

				// もぐら出現
				case JOTAI_MOGURA:

					// 敵を描画する
					g.drawImage(imageTekis[0], xTekis[i], yTekis[i], 100, 100, this);

					// ブレイク
					break;

				// やられた
				case JOTAI_YARARETA:

					// 敵を描画する
					g.drawImage(imageTekis[1], xTekis[i], yTekis[i], 100, 100, this);

					// ブレイク
					break;

				// 画びょう
				case JOTAI_GABYO:

					// 敵を描画する
					g.drawImage(imageTekis[2], xTekis[i], yTekis[i], 100, 100, this);

					// ブレイク
					break;

				// 岩
				case JOTAI_ROCK:

					// 敵を描画する
					g.drawImage(imageTekis[3], xTekis[i], yTekis[i], 100, 100, this);

					// ブレイク
					break;

				// デフォルト
				default:

					// ブレイク
					break;

			} // end switch 状態により処理を分ける

		} // end for ループ

		// うししはいる？
		if(timeUsisi > 0){

			// 時間－１
			timeUsisi--;

			// うししを描画する
			g.drawImage(imageUsisi, 650 + (int)(((timeUsisi < 150)?0: timeUsisi - 150) * 4.5 + ((timeUsisi < 30)?30 - timeUsisi: 0) * 4.5) ,
									yUsisi + ((timeUsisi % 10 < 5)?0: 2), 150, 132, this);

		} // end if うししはいる？

		// ふつう or たたいて いる場合、
		if(jotaiPH == JOTAI_PH_FUTU || jotaiPH == JOTAI_PH_TATAKU){

			// ピコピコハンマーを描画する
			g.drawImage(imagePHs[jotaiPH], px, py, 100, 88, this);

		// 画びょうの場合、
		}else if(jotaiPH == JOTAI_PH_GABYO){

			// ピコピコハンマーを描画する
			g.drawImage(imagePHs[jotaiPH], px, py, 100, 88, this);

		// 岩の場合、
		}else if(jotaiPH == JOTAI_PH_ROCK){

			// ピコピコハンマーを描画する
			g.drawImage(imagePHs[JOTAI_PH_TATAKU], px + (timePH % 4) * 5 - 10, py, 100, 88, this);

		}

		// 得点を表示する
		g.setColor(Color.yellow);
		g.setFont(fontScore);
		g.drawString("得点：" + score, 0, 24);

		// ゲーム中の場合、
		if(phase == PHASE_GAME || phase == PHASE_EXTEND_GAME){

			// 制限時間を表示する
			g.setColor(Color.red);
			g.setFont(fontTime);
			g.drawString("残り時間：" + time / 60, 300, 24);

			// 延長戦ゲーム中の場合、
			if(phase == PHASE_EXTEND_GAME){

				// 延長戦を表示する
				g.setColor(new Color((int)(Math.random() * 16777216)));
				g.drawString("延長戦", 550, 24);

			}

		// 延長戦の場合、
		}else if(phase == PHASE_EXTEND){

			g.setColor(Color.red);
			g.setFont(fontExtraGame);
			g.drawString("延長戦突入", 240, 300);

		// ゲームオーバーの場合、
		}else if(phase == PHASE_GAMEOVER){
			g.setColor(Color.lightGray);
			g.setFont(fontGameOver);
			g.drawString("ゲームオーバー", 240, 300);

			// コメント
			if(score >= 100){
				g.drawString("すごい！すごい！", 220, 350);
			}else if(score >= 80){
				g.drawString("かなりの腕前じゃ", 220, 350);
			}else if(score >= 50){
				g.drawString("おぬしやるのう", 240, 350);
			}else if(score >= 40){
				g.drawString("もう一歩じゃな", 240, 350);
			}else if(score >= 20){
				g.drawString("まだまだじゃな", 240, 350);
			}else{
				g.drawString("がんばるのじゃ", 240, 350);
			}

		} // end if ゲームオーバーの場合

	} // end paint

	/**
	 * マウスアダプタ
	 */
	private class MGMouseAdapter extends MouseAdapter {

		/**
		 * マウスが押された時に呼ばれます
		 */
		public void mousePressed(MouseEvent me){

			// ゲーム中の場合、
			if(phase == PHASE_GAME || phase == PHASE_EXTEND_GAME){

				// 普通の場合、
				if(jotaiPH == JOTAI_PH_FUTU){

					// ピコピコハンマー叩く
					jotaiPH = JOTAI_PH_TATAKU;
					// 場所を記憶する
					px = me.getX() - 100;
					py = me.getY() - 100;

					// 当たったフラグ
					boolean flgAtari = false;

					// ループ
					for(int i = 0; i < 10; i++){

						// いない場合、
						if(jotaiTekis[i] == JOTAI_NASI){
							// コンティニュー
							continue;
						}

						// 敵との当たり判定
						if(px > xTekis[i] - 50 && px < xTekis[i] + 90 && py > yTekis[i] - 70 && py < yTekis[i] + 60){

							// もぐらの場合、
							if(jotaiTekis[i] == JOTAI_MOGURA){

								// 当たった
								flgAtari = true;
								// やられた
								jotaiTekis[i] = JOTAI_YARARETA;
								timeTekis[i] = 30;
								score++;
								// 効果音をスタート
								wavePicoHammerHit.start();

							// 画びょうの場合、
							}else if(jotaiTekis[i] == JOTAI_GABYO){

								// 当たった
								flgAtari = true;
								// ピコピコハンマー画びょう
								jotaiPH = JOTAI_PH_GABYO;
								// 時間
								timePH = 60;
								timeUsisi = 180;
								// うしし位置を決定
								yUsisi = py;
								// 効果音をスタート
								wavePicoHammerGabyo.start();

							// 岩の場合、
							}else if(jotaiTekis[i] == JOTAI_ROCK){

								// 当たった
								flgAtari = true;
								// ピコピコハンマー岩
								jotaiPH = JOTAI_PH_ROCK;
								// 時間
								timePH = 60;
								timeUsisi = 180;
								// うしし位置を決定
								yUsisi = py;
								// 効果音をスタート
								wavePicoHammerRock.start();

							} // end if

						} // end if 敵との当たり判定

					} // end for ループ

					// 外れた場合、
					if(!flgAtari){
						// 効果音をスタート
						wavePicoHammerMiss.start();
					}

				} // end if 普通の場合

			} // end if ゲーム中の場合

		} //  end mousePressed

		/**
		 * マウスが離された時に呼ばれます
		 */
		public void mouseReleased(MouseEvent me){

			// 叩いていた場合、
			if(jotaiPH == JOTAI_PH_TATAKU){

				// ピコピコハンマーを上げる
				jotaiPH = JOTAI_PH_FUTU;
				// 場所を記憶する
				px = me.getX() - 100;
				py = me.getY() - 100;

			}

		} //  end mouseReleased

		/**
		 * マウスが移動された時に呼ばれます
		 */
		public void mouseMoved(MouseEvent me){

			// 普通 or 叩いた 場合、
			if(jotaiPH == JOTAI_PH_FUTU || jotaiPH == JOTAI_PH_TATAKU){

				// 場所を記憶する
				px = me.getX() - 100;
				py = me.getY() - 100;

			}

		} //  end mouseMoved

		/**
		 * マウスがドラッグされた時に呼ばれます
		 */
		public void mouseDragged(MouseEvent me){

			// 普通 or 叩いた 場合、
			if(jotaiPH == JOTAI_PH_FUTU || jotaiPH == JOTAI_PH_TATAKU){

				// 場所を記憶する
				px = me.getX() - 100;
				py = me.getY() - 100;

			}

		} //  end mouseDragged

	} // end MGMouseAdapter

	/**
	 * タイマークラス.
	 */
	private class TimerActionListener extends TimerTask {

		public void run(){

			// 実行メソッド呼び出し
			MGPanel.this.run();

			// 描画する
			repaint();

		} // end actionPerformed

	} // end TimerActionListener

}
