import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.imageio.*;
import javax.sound.sampled.*;

/**
 * タイトルクラス
 * @author Aki2
 * @author KIN
 * <pre>
 * タイトルクラス。
 * アニメーションgifとBGMをコラボしてみました。
 * Aki2氏が土台を作って、KINが修正を行っています。
 * </pre>
 */
public class MGTitle {

	/** ピコピコハンマーの状態 */
	private final int				JOTAI_PH_FUTU		= 0;
	private final int				JOTAI_PH_TATAKU		= 1;

	/** パネルクラス */
	private		MGPanel				panel				= null;

	/** マウスアダプタ */
	private		MGMouseAdapter		mgma				= null;

	/** タイマー */
	private		java.util.Timer		timerThis			= null;

	/** 時間制御 */
	public		int					ctrlTime			= 0;

	/** イメージ */
	private		Image				imgBackYard			= null;
	private		Image				imgMogLogo			= null;
	private		Image				imgMogTalk			= null;
	private		ArrayList<BufferedImage>	imgMogRightTbl	= null;
	private		ArrayList<BufferedImage>	imgMogLeftTbl	= null;
	private		ArrayList<BufferedImage>	imgMogCenterTbl	= null;
	private		Image				imgMogHole			= null;
	private		Image				imgMogUta			= null;
	private		Image				imgMogHoleMini		= null;

	/** イメージポジション */
	private		int					posRight			= 0;
	private		int					posLeft				= 0;
	private		int					posCenter			= 0;

	/** ピコピコハンマー座標 */
	private		int					px					= 0;
	/** ピコピコハンマー座標 */
	private		int					py					= 0;

	/** ピコピコハンマー */
	private		BufferedImage[]		imagePHs			= null;
	/** ピコピコハンマー状態 */
	private		int					jotaiPH				= JOTAI_PH_FUTU;

	/** BGM */
	private		KSoundWave			waveTitle			= null;

	/** 効果音 */
	private		KSoundWave			waveCredit			= null;

	/** フォント */
	private		Font				fontLogo1			= new Font("HGP創英角ﾎﾟｯﾌﾟ体", Font.BOLD | Font.ITALIC, 144);
	private		Font				fontLogo2			= new Font("HGP創英角ﾎﾟｯﾌﾟ体", Font.BOLD | Font.ITALIC, 160);

	/** ロゴ時間 */
	private		int					timeLogo			= 0;

	/**
	 * コンストラクタ
	 */
	public MGTitle(MGPanel panel) throws IOException{

		// パネルクラスを保持
		this.panel = panel;

		//
		// イメージを生成
		//

		// 先にもぐら達を読み込む
		// 事前読込が可能のようなため
		// アニメgifの制御は、アプリで行う。ちらつき防止
		imgMogCenterTbl = getImages("image\\MogCenter.gif");
		imgMogLeftTbl = getImages("image\\MogLeft.gif");
		imgMogRightTbl = getImages("image\\MogRight.gif");

		// その他の静的イメージ
		imgBackYard = panel.getToolkit().getImage("image\\MogBackYard.jpg");
		imgMogLogo = panel.getToolkit().getImage("image\\MogLogo.gif");
		imgMogHole = panel.getToolkit().getImage("image\\MogHole.gif");
		imgMogHoleMini = panel.getToolkit().getImage("image\\MogHoleMini.gif");

		// ピコピコハンマーを読み込む
		imagePHs = new BufferedImage[2];
		InputStream isPH00 = this.getClass().getResourceAsStream("image\\PH00.gif");
		imagePHs[0] = ImageIO.read(isPH00);
		isPH00.close();

		InputStream isPH01 = this.getClass().getResourceAsStream("image\\PH01.gif");
		imagePHs[1] = ImageIO.read(isPH01);
		isPH01.close();

		// BGMを生成
		waveTitle = new KSoundWave(panel, "sound\\MGDance.wav", true);

		// 効果音を生成
		waveCredit = new KSoundWave(this, "sound\\Credit.wav", false);

	} // end MGTitle

	/**
	 * 初期化
	 */
	public void init(){

		// 初期化
		ctrlTime = 0;

		// 動的イメージ
		// アニメgifの制御は、システムに任せる。ちらつき気にならないため
		imgMogTalk = panel.getToolkit().getImage("image\\MogTalk.gif");
		imgMogUta = panel.getToolkit().getImage("image\\MogUta.gif");

		// マウスアダプタを生成
		mgma = new MGMouseAdapter();
		// パネルにマウスリスナーを追加する
		panel.addMouseListener(mgma);
		panel.addMouseMotionListener(mgma);

	} // end init

	/**
	 * 初期化
	 */
	public void end(){

		// 動的イメージのリソースを開放する
		imgMogTalk.flush();
		imgMogUta.flush();

		imgMogTalk = null;
		imgMogUta = null;

		// BGMをストップ
		waveTitle.stop();

		// 効果音をスタート
		waveCredit.start();

		// マウスリスナーを削除する
		panel.removeMouseListener(mgma);
		panel.removeMouseMotionListener(mgma);

		// ゲームスタート
		panel.start();

	}

	/**
	 * 実行
	 */
	public void run(){

		// 初めての実行の場合、
		if(ctrlTime == 0){

			// 実行可能状態へ
			ctrlTime = 1;

			// BGMをスタート
			waveTitle.start();

		} // end if 初めての実行の場合

	} // end run

	/**
	 * 描画
	 */
	public void paint(Graphics g){

		// 実行可能の状態の場合、
		if(ctrlTime == 1){

			// 静止画とGIFアニメの表示　下になる絵から順番に表示
			g.drawImage(imgBackYard, 0, 0, null);
			g.drawImage(imgMogLogo, 165, 10, null);

			// ロゴ時間＋１
			timeLogo++;

			// 普通ロゴ
			if(timeLogo <= 10){

				g.setFont(fontLogo1);

				// 当たり判定
				if(px > 180 && px < 580 && py > 80 && py < 250){
					g.setColor(new Color((int)(Math.random() * 16777216)));
				}else{
					g.setColor(Color.orange);
				}

				g.drawString("モグラ", 170, 220);

			// 大きいロゴ
			}else{

				g.setFont(fontLogo2);

				// 当たり判定
				if(px > 180 && px < 580 && py > 80 && py < 250){
					g.setColor(new Color((int)(Math.random() * 128 + 128) ,(int)(Math.random() * 128 + 128), (int)(Math.random() * 128 + 128)));
				}else{
					g.setColor(Color.yellow);
				}

				g.drawString("モグラ", 150, 230);

				// ロゴ時間が過ぎた場合、
				if(timeLogo > 13){
					// ロゴ時間を戻す
					timeLogo = 0;
				}

			} // end if

			g.drawImage(imgMogTalk, 0, 100, null);

			posRight++;
			posRight = (posRight < imgMogRightTbl.size())?posRight: 0;
			g.drawImage(imgMogRightTbl.get(posRight), 560, 263, 200, 200, null);

			posLeft++;
			posLeft = (posLeft < imgMogLeftTbl.size())?posLeft: 0;
			g.drawImage(imgMogLeftTbl.get(posLeft), 5, 240, 200, 200, null);

			posCenter++;
			posCenter = (posCenter < imgMogCenterTbl.size())?posCenter: 0;
			g.drawImage(imgMogCenterTbl.get(posCenter), 180, 205, null);

			g.drawImage(imgMogHole, 160, 410, 410, 133, null);
			g.drawImage(imgMogUta, 1, 500, null);
			g.drawImage(imgMogHoleMini, 570, 353, null);
			g.drawImage(imgMogHoleMini, 10, 375, null);

			// ピコピコハンマーを描画する
			g.drawImage(imagePHs[jotaiPH], px, py, 50, 44, panel);

		} // end if 実行可能の状態の場合

	} // end paint

	/**
	 * イメージ取得
	 * <pre>
	 * 複数イメージをArrayListとして取得します。
	 * </pre>
	 * @return ArrayList<BufferedImage> イメージ一覧
	 */
	public ArrayList<BufferedImage> getImages(String filePath) throws IOException{

		ImageReader reader = null;

		// ImageReaderを取得
		Iterator<ImageReader> itr = ImageIO.getImageReadersByFormatName("gif");

		// イメージ一覧を生成
		ArrayList<BufferedImage> images = new ArrayList<BufferedImage>();

		if(itr.hasNext()){
			reader = itr.next();
		}else{
			throw new RuntimeException();
		}

		// ストリームを取得
		InputStream isFile = this.getClass().getResourceAsStream(filePath);

		// ImageInputStreamを取得し、ImageReaderに設定する
		reader.setInput(ImageIO.createImageInputStream(isFile));

		// サムネールを除く使用可能なイメージの数を取得
		int count = reader.getNumImages(true);

		// イメージ数分、ループ
		for(int i = 0; i < count; i++) {
			// イメージ一覧に追加する
		    images.add(reader.read(i));
		}

		// ストリームを閉じる
		isFile.close();

		// リターン
		return images;

	} // end getImages

	/**
	 * マウスアダプタ
	 */
	private class MGMouseAdapter extends MouseAdapter {

		/**
		 * マウスが押された時に呼ばれます
		 */
		public void mousePressed(MouseEvent me){

			// 普通の場合、
			if(jotaiPH == JOTAI_PH_FUTU){

				// ピコピコハンマー叩く
				jotaiPH = JOTAI_PH_TATAKU;
				// 場所を記憶する
				px = me.getX() - 50;
				py = me.getY() - 50;

			} // end if 普通の場合

		} //  end mousePressed

		/**
		 * マウスが離された時に呼ばれます
		 */
		public void mouseReleased(MouseEvent me){

			// 叩いていた場合、
			if(jotaiPH == JOTAI_PH_TATAKU){

				// ピコピコハンマーを上げる
				jotaiPH = JOTAI_PH_FUTU;
				// 場所を記憶する
				px = me.getX() - 50;
				py = me.getY() - 50;

				// 当たり判定
				if(px > 180 && px < 580 && py > 80 && py < 250){

					// 終了メソッド呼び出し
					MGTitle.this.end();

				} // end if 当たり判定

			}

		} //  end mouseReleased

		/**
		 * マウスが移動された時に呼ばれます
		 */
		public void mouseMoved(MouseEvent me){

			// 場所を記憶する
			px = me.getX() - 50;
			py = me.getY() - 50;

		} //  end mouseMoved

		/**
		 * マウスがドラッグされた時に呼ばれます
		 */
		public void mouseDragged(MouseEvent me){

			// 場所を記憶する
			px = me.getX() - 50;
			py = me.getY() - 50;

		} //  end mouseDragged

	} // end MGMouseAdapter

}
